@@ -51,7 +51,7 @@ use Bugzilla::Util;
 Exporter::export_ok_tags('admin', 'db');
 
 # Bugzilla version
-$Bugzilla::Config::VERSION = "2.17.3";
+$Bugzilla::Config::VERSION = "2.17.4";
 
 use Safe;
 
@@ -171,8 +171,8 @@ sub UpdateParams {
     delete $param{'version'};
 
     # Change from a boolean for quips to multi-state
-    if (exists $param{'usequip'} && !exists $param{'allowquips'}) {
-        $param{'allowquips'} = $param{'usequip'} ? 'on' : 'off';
+    if (exists $param{'usequip'} && !exists $param{'enablequips'}) {
+        $param{'enablequips'} = $param{'usequip'} ? 'on' : 'off';
         delete $param{'usequip'};
     }
 
@@ -186,6 +186,11 @@ sub UpdateParams {
         $param{'useentrygroupdefault'} = $param{'usebuggroupsentry'};
     }
 
+    # Modularise auth code
+    if (exists $param{'useLDAP'} && !exists $param{'loginmethod'}) {
+        $param{'loginmethod'} = $param{'useLDAP'} ? "LDAP" : "DB";
+    }
+
     # --- DEFAULTS FOR NEW PARAMS ---
 
     foreach my $item (@param_list) {
