@@ -72,7 +72,7 @@ sub check_shadowdb {
         return "You need to specify a host when using a shadow database";
     }
 
-    # Can't test existance of this because ConnectToDatabase uses the param,
+    # Can't test existence of this because ConnectToDatabase uses the param,
     # but we can't set this before testing....
     # This can really only be fixed after we can use the DBI more openly
     return "";
@@ -99,8 +99,8 @@ sub check_webdotbase {
         # Check .htaccess allows access to generated images
         if(-e "data/webdot/.htaccess") {
             open HTACCESS, "data/webdot/.htaccess";
-            if(! grep(/png/,<HTACCESS>)) {
-                return "Dependency graph images are not accessible.\nDelete data/webdot/.htaccess and re-run checksetup.pl to rectify.\n";
+            if(! grep(/ \\\.png\$/,<HTACCESS>)) {
+                return "Dependency graph images are not accessible.\nAssuming that you have not modified the file, delete data/webdot/.htaccess and re-run checksetup.pl to rectify.\n";
             }
             close HTACCESS;
         }
@@ -123,6 +123,31 @@ sub check_netmask {
     return "";
 }
 
+sub check_loginmethod {
+    # doeditparams traverses the list of params, and for each one it checks,
+    # then updates. This means that if one param checker wants to look at 
+    # other params, it must be below that other one. So you can't have two 
+    # params mutually dependant on each other.
+    # This means that if someone clears the LDAP config params after setting
+    # the login method as LDAP, we won't notice, but all logins will fail.
+    # So don't do that.
+
+    my ($method, $entry) = @_;
+    my $res = check_multi($method, $entry);
+    return $res if $res;
+    if ($method eq 'DB') {
+        # No params
+    } elsif ($method eq 'LDAP') {
+        eval "require Net::LDAP";
+        return "Error requiring Net::LDAP: '$@'" if $@;
+        return "LDAP servername is missing" unless Param("LDAPserver");
+        return "LDAPBaseDN is empty" unless Param("LDAPBaseDN");
+    } else {
+        return "Unknown loginmethod '$method' in check_loginmethod";
+    }
+    return "";
+}
+
 # OK, here are the parameter definitions themselves.
 #
 # Each definition is a hash with keys:
@@ -201,6 +226,26 @@ sub check_netmask {
    checker => \&check_urlbase
   },
 
+  {
+   name => 'languages' ,
+   desc => 'A comma-separated list of RFC 1766 language tags. These ' .
+           'identify the languages in which you wish Bugzilla output ' .
+           'to be displayed. Note that you must install the appropriate ' .
+           'language pack before adding a language to this Param. The ' .
+           'language used is the one in this list with the highest ' .
+           'q-value in the user\'s Accept-Language header.' ,
+   type => 't' ,
+   default => 'en'
+  },
+
+  {
+   name => 'defaultlanguage',
+   desc => 'The UI language Bugzilla falls back on if no suitable ' .
+           'language is found in the user\'s Accept-Language header.' ,
+   type => 't' ,
+   default => 'en'
+  },
+
   {
    name => 'cookiepath',
    desc => 'Path, relative to your web document root, to which to restrict ' .
@@ -226,11 +271,12 @@ sub check_netmask {
    name => 'enablequips',
    desc => 'Controls the appearance of quips at the top of buglists.<ul> ' .
            '<li>on - Bugzilla will display a quip, and lets users add to ' .
-           'the list.</li><li>frozen - Bugzilla will display a quip but ' .
-           'not permit new additions.</li><li>off - Bugzilla will not ' .
-           'display quips.</li></ul>',
+           'the list.</li><li>approved - quips can be entered, but need ' .
+           'be approved before shown</li><li>frozen - Bugzilla will display ' .
+           'a quip but not permit new additions.</li><li>off - Bugzilla ' .
+           'will not display quips.</li></ul>',
    type => 's',
-   choices => ['on','frozen','off'],
+   choices => ['on', 'approved', 'frozen', 'off'],
    default => 'on',
    checker => \&check_multi
   },
@@ -294,23 +340,13 @@ sub check_netmask {
            'This is done so that long slow read-only operations can be used ' .
            'against this db, and not lock up things for everyone else. This ' .
            'database is on the <tt>shadowdbhost</tt>, and must exist. ' .
-           'Bugzilla does not update it, if you use this paramater, then ' .
+           'Bugzilla does not update it, if you use this parameter, then ' .
            'you need to set up replication for your database',
    type => 't',
    default => '',
    checker => \&check_shadowdb
   },
 
-  {
-   name => 'useLDAP',
-   desc => 'Turn this on to use an LDAP directory for user authentication ' .
-           'instead of the Bugzilla database.  (User profiles will still be ' .
-           'stored in the database, and will match against the LDAP user by ' .
-           'email address.)',
-   type => 'b',
-   default => 0
-  },
-
   {
    name => 'LDAPserver',
    desc => 'The name (and optionally port) of your LDAP server. (e.g. ' .
@@ -319,6 +355,16 @@ sub check_netmask {
    default => ''
   },
 
+  {
+   name => 'LDAPbinddn',
+   desc => 'If your LDAP server requires that you use a binddn and password ' .
+           'instead of binding anonymously, enter it here ' .
+           '(e.g. cn=default,cn=user:password). ' .
+           'Leave this empty for the normal case of an anonymous bind.',
+   type => 't',
+   default => ''
+  },
+
   {
    name => 'LDAPBaseDN',
    desc => 'The BaseDN for authenticating users against. (e.g. ' .
@@ -327,6 +373,13 @@ sub check_netmask {
    default => ''
   },
 
+  {
+   name => 'LDAPuidattribute',
+   desc => 'The name of the attribute containing the user\'s login name.',
+   type => 't',
+   default => 'uid'
+  },
+
   {
    name => 'LDAPmailattribute',
    desc => 'The name of the attribute of a user in your directory that ' .
@@ -335,6 +388,29 @@ sub check_netmask {
    default => 'mail'
   },
 
+  {
+   name => 'loginmethod',
+   desc => 'The type of login authentication to use:
+            <dl>
+              <dt>DB</dt>
+              <dd>
+                Bugzilla\'s builtin authentication. This is the most common
+                choice.
+              </dd>
+              <dt>LDAP</dt>
+              <dd>
+                LDAP authentication using an LDAP server. This method is
+                experimental; please see the Bugzilla documentation for more
+                information. Using this method requires additional parameters
+                to be set above.
+              </dd>
+             </dl>',
+   type => 's',
+   choices => [ 'DB', 'LDAP' ],
+   default => 'DB',
+   checker => \&check_loginmethod
+  },
+
   {
    name => 'mostfreqthreshold',
    desc => 'The minimum number of duplicates a bug needs to show up on the ' .
@@ -423,8 +499,8 @@ X-Bugzilla-Reason: %reasonsheader%
 
 %diffs%
 
-
-
+--%space%
+Configure bugmail: %urlbase%userprefs.cgi?tab=email
 %reasonsbody%'
   },
 
