@@ -28,6 +28,7 @@
 
 use strict;
 
+use Bugzilla::DB qw(:DEFAULT :deprecated);
 use Bugzilla::Constants;
 use Bugzilla::Util;
 # Bring ChmodDataFile in until this is all moved to the module
@@ -97,7 +98,6 @@ $::SIG{PIPE} = 'IGNORE';
 
 $::defaultqueryname = "(Default query)"; # This string not exposed in UI
 $::unconfirmedstate = "UNCONFIRMED";
-$::dbwritesallowed = 1;
 
 #sub die_with_dignity {
 #    my ($err_msg) = @_;
@@ -106,175 +106,6 @@ $::dbwritesallowed = 1;
 #}
 #$::SIG{__DIE__} = \&die_with_dignity;
 
-sub ConnectToDatabase {
-    my ($useshadow) = (@_);
-    $::dbwritesallowed = !$useshadow;
-    $useshadow &&= Param("shadowdb");
-    my $connectstring;
-
-    if ($useshadow) {
-        if (defined $::shadow_dbh) {
-            $::db = $::shadow_dbh;
-            return;
-        }
-        $connectstring="DBI:mysql:host=" . Param("shadowdbhost") .
-          ";database=" . Param('shadowdb') . ";port=" . Param("shadowdbport");
-        if (Param("shadowdbsock") ne "") {
-            $connectstring .= ";mysql_socket=" . Param("shadowdbsock");
-        }
-    } else {
-        if (defined $::main_dbh) {
-            $::db = $::main_dbh;
-            return;
-        }
-        $connectstring="DBI:mysql:host=$::db_host;database=$::db_name;port=$::db_port";
-        if ($::db_sock ne "") {
-            $connectstring .= ";mysql_socket=$::db_sock";
-        }
-    }
-    $::db = DBI->connect($connectstring, $::db_user, $::db_pass)
-      || die "Bugzilla is currently broken. Please try again " .
-        "later. If the problem persists, please contact " .
-        Param("maintainer") . ". The error you should quote is: " .
-        $DBI::errstr;
-
-    if ($useshadow) {
-        $::shadow_dbh = $::db;
-    } else {
-        $::main_dbh = $::db;
-    }
-}
-
-sub ReconnectToShadowDatabase {
-    if (Param("shadowdb")) {
-        ConnectToDatabase(1);
-    }
-}
-
-sub ReconnectToMainDatabase {
-    if (Param("shadowdb")) {
-        ConnectToDatabase();
-    }
-}
-
-# This is used to manipulate global state used by SendSQL(),
-# MoreSQLData() and FetchSQLData().  It provides a way to do another
-# SQL query without losing any as-yet-unfetched data from an existing
-# query.  Just push the current global state, do your new query and fetch
-# any data you need from it, then pop the current global state.
-# 
-@::SQLStateStack = ();
-
-sub PushGlobalSQLState() {
-    push @::SQLStateStack, $::currentquery;
-    push @::SQLStateStack, [ @::fetchahead ]; 
-}
-
-sub PopGlobalSQLState() {
-    die ("PopGlobalSQLState: stack underflow") if ( $#::SQLStateStack < 1 );
-    @::fetchahead = @{pop @::SQLStateStack};
-    $::currentquery = pop @::SQLStateStack;
-}
-
-sub SavedSQLStates() {
-    return ($#::SqlStateStack + 1) / 2;
-}
-
-
-my $dosqllog = (-e "data/sqllog") && (-w "data/sqllog");
-
-sub SqlLog {
-    if ($dosqllog) {
-        my ($str) = (@_);
-        open(SQLLOGFID, ">>data/sqllog") || die "Can't write to data/sqllog";
-        if (flock(SQLLOGFID,2)) { # 2 is magic 'exclusive lock' const.
-
-            # if we're a subquery (ie there's pushed global state around)
-            # indent to indicate the level of subquery-hood
-            #
-            for (my $i = SavedSQLStates() ; $i > 0 ; $i--) {
-                print SQLLOGFID "\t";
-            }
-
-            print SQLLOGFID time2str("%D %H:%M:%S $$", time()) . ": $str\n";
-        }
-        flock(SQLLOGFID,8);     # '8' is magic 'unlock' const.
-        close SQLLOGFID;
-    }
-}
-
-sub SendSQL {
-    my ($str) = (@_);
-
-    # Don't use DBI's taint stuff yet, because:
-    # a) We don't want out vars to be tainted (yet)
-    # b) We want to know who called SendSQL...
-    # Is there a better way to do b?
-    if (is_tainted($str)) {
-        die "Attempted to send tainted string '$str' to the database";
-    }
-
-    my $iswrite =  ($str =~ /^(INSERT|REPLACE|UPDATE|DELETE)/i);
-    if ($iswrite && !$::dbwritesallowed) {
-        die "Evil code attempted to write '$str' to the shadow database";
-    }
-
-    # If we are shutdown, we don't want to run queries except in special cases
-    if (Param('shutdownhtml')) {
-        if ($0 =~ m:[\\/]((do)?editparams.cgi)$:) {
-            $::ignorequery = 0;
-        } else {
-            $::ignorequery = 1;
-            return;
-        }
-    }
-    SqlLog($str);
-    $::currentquery = $::db->prepare($str);
-    if (!$::currentquery->execute) {
-        my $errstr = $::db->errstr;
-        # Cut down the error string to a reasonable.size
-        $errstr = substr($errstr, 0, 2000) . ' ... ' . substr($errstr, -2000)
-                if length($errstr) > 4000;
-        die "$str: " . $errstr;
-    }
-    SqlLog("Done");
-}
-
-sub MoreSQLData {
-    # $::ignorequery is set in SendSQL
-    if ($::ignorequery) {
-        return 0;
-    }
-    if (defined @::fetchahead) {
-        return 1;
-    }
-    if (@::fetchahead = $::currentquery->fetchrow_array) {
-        return 1;
-    }
-    return 0;
-}
-
-sub FetchSQLData {
-    # $::ignorequery is set in SendSQL
-    if ($::ignorequery) {
-        return;
-    }
-    if (defined @::fetchahead) {
-        my @result = @::fetchahead;
-        undef @::fetchahead;
-        return @result;
-    }
-    return $::currentquery->fetchrow_array;
-}
-
-
-sub FetchOneColumn {
-    my @row = FetchSQLData();
-    return $row[0];
-}
-
-    
-
 @::default_column_list = ("bug_severity", "priority", "rep_platform", 
                           "assigned_to", "bug_status", "resolution",
                           "short_short_desc");
@@ -296,7 +127,6 @@ sub AppendComment {
        # zero or more digits, OR we have a period followed by one or more digits
        if ($work_time !~ /^-?(?:\d+(?:\.\d*)?|\.\d+)$/) { 
           ThrowUserError("need_numeric_value");
-          return;
        }
     } else { $work_time = 0 };
 
@@ -407,19 +237,21 @@ sub GenerateVersionTable {
 
     my @list = sort { uc($a) cmp uc($b)} keys(%::versions);
     @::legal_product = @list;
-    my $tmpname = "data/versioncache.$$";
-    open(FID, ">$tmpname") || die "Can't create $tmpname";
 
-    print FID "#\n";
-    print FID "# DO NOT EDIT!\n";
-    print FID "# This file is automatically generated at least once every\n";
-    print FID "# hour by the GenerateVersionTable() sub in globals.pl.\n";
-    print FID "# Any changes you make will be overwritten.\n";
-    print FID "#\n";
+    require File::Temp;
+    my ($fh, $tmpname) = File::Temp::tempfile("versioncache.XXXXX",
+                                              DIR => "data");
+
+    print $fh "#\n";
+    print $fh "# DO NOT EDIT!\n";
+    print $fh "# This file is automatically generated at least once every\n";
+    print $fh "# hour by the GenerateVersionTable() sub in globals.pl.\n";
+    print $fh "# Any changes you make will be overwritten.\n";
+    print $fh "#\n";
 
     require Data::Dumper;
-    print FID Data::Dumper->Dump([\@::log_columns, \%::versions],
-                                 ['*::log_columns', '*::versions']);
+    print $fh (Data::Dumper->Dump([\@::log_columns, \%::versions],
+                                  ['*::log_columns', '*::versions']));
 
     foreach my $i (@list) {
         if (!defined $::components{$i}) {
@@ -427,23 +259,23 @@ sub GenerateVersionTable {
         }
     }
     @::legal_versions = sort {uc($a) cmp uc($b)} keys(%varray);
-    print FID Data::Dumper->Dump([\@::legal_versions, \%::components],
-                                 ['*::legal_versions', '*::components']);
+    print $fh (Data::Dumper->Dump([\@::legal_versions, \%::components],
+                                  ['*::legal_versions', '*::components']));
     @::legal_components = sort {uc($a) cmp uc($b)} keys(%carray);
 
-    print FID Data::Dumper->Dump([\@::legal_components, \@::legal_product,
-                                  \@::legal_priority, \@::legal_severity,
-                                  \@::legal_platform, \@::legal_opsys,
-                                  \@::legal_bug_status, \@::legal_resolution],
-                                 ['*::legal_components', '*::legal_product',
-                                  '*::legal_priority', '*::legal_severity',
-                                  '*::legal_platform', '*::legal_opsys',
-                                  '*::legal_bug_status', '*::legal_resolution']);
+    print $fh (Data::Dumper->Dump([\@::legal_components, \@::legal_product,
+                                   \@::legal_priority, \@::legal_severity,
+                                   \@::legal_platform, \@::legal_opsys,
+                                   \@::legal_bug_status, \@::legal_resolution],
+                                  ['*::legal_components', '*::legal_product',
+                                   '*::legal_priority', '*::legal_severity',
+                                   '*::legal_platform', '*::legal_opsys',
+                                   '*::legal_bug_status', '*::legal_resolution']));
 
-    print FID Data::Dumper->Dump([\@::settable_resolution, \%::proddesc,
-                                  \@::enterable_products, \%::prodmaxvotes],
-                                 ['*::settable_resolution', '*::proddesc',
-                                  '*::enterable_products', '*::prodmaxvotes']);
+    print $fh (Data::Dumper->Dump([\@::settable_resolution, \%::proddesc,
+                                   \@::enterable_products, \%::prodmaxvotes],
+                                  ['*::settable_resolution', '*::proddesc',
+                                   '*::enterable_products', '*::prodmaxvotes']));
 
     if ($dotargetmilestone) {
         # reading target milestones in from the database - matthew@zeroknowledge.com
@@ -466,12 +298,12 @@ sub GenerateVersionTable {
             }
         }
 
-        print FID Data::Dumper->Dump([\%::target_milestone,
-                                      \@::legal_target_milestone,
-                                      \%::milestoneurl],
-                                     ['*::target_milestone',
-                                      '*::legal_target_milestone',
-                                      '*::milestoneurl']);
+        print $fh (Data::Dumper->Dump([\%::target_milestone,
+                                       \@::legal_target_milestone,
+                                       \%::milestoneurl],
+                                      ['*::target_milestone',
+                                       '*::legal_target_milestone',
+                                       '*::milestoneurl']));
     }
 
     SendSQL("SELECT id, name FROM keyworddefs ORDER BY name");
@@ -482,11 +314,11 @@ sub GenerateVersionTable {
         $::keywordsbyname{$name} = $id;
     }
 
-    print FID Data::Dumper->Dump([\@::legal_keywords, \%::keywordsbyname],
-                                 ['*::legal_keywords', '*::keywordsbyname']);
+    print $fh (Data::Dumper->Dump([\@::legal_keywords, \%::keywordsbyname],
+                                  ['*::legal_keywords', '*::keywordsbyname']));
 
-    print FID "1;\n";
-    close FID;
+    print $fh "1;\n";
+    close $fh;
 
     rename $tmpname, "data/versioncache" || die "Can't rename $tmpname to versioncache";
     ChmodDataFile('data/versioncache', 0666);
@@ -521,7 +353,7 @@ sub GetVersionTable {
     }
     if (time() - $mtime > 3600) {
         use Token;
-        Token::CleanTokenTable() if $::dbwritesallowed;
+        Token::CleanTokenTable() if Bugzilla->dbwritesallowed;
         GenerateVersionTable();
     }
     require 'data/versioncache';
@@ -643,12 +475,15 @@ sub AnyEntryGroups {
     $query .= " AND product_id = $product_id" if ($product_id);
     $query .= " LIMIT 1";
     SendSQL($query);
-    $::CachedAnyEntryGroups{$product_id} = MoreSQLData();
-    FetchSQLData();
-    PopGlobalSQLState();
-    return $::CachedAnyEntryGroups{$product_id};
+    if (MoreSQLData()) {
+       $::CachedAnyEntryGroups{$product_id} = MoreSQLData();
+       FetchSQLData();
+       PopGlobalSQLState();
+       return $::CachedAnyEntryGroups{$product_id};
+    } else {
+       return undef;
+    }
 }
-
 #
 # This function checks if there are any default groups defined.
 # If so, then groups may have to be changed when bugs move from
@@ -714,9 +549,15 @@ sub CanEnterProduct {
 
 #
 # This function returns an alphabetical list of product names to which
-# the user can enter bugs.
+# the user can enter bugs.  If the $by_id parameter is true, also retrieves IDs
+# and pushes them onto the list as id, name [, id, name...] for easy slurping
+# into a hash by the calling code.
 sub GetSelectableProducts {
-    my $query = "SELECT name " .
+    my ($by_id) = @_;
+
+    my $extra_sql = $by_id ? "id, " : "";
+
+    my $query = "SELECT $extra_sql name " .
                 "FROM products " .
                 "LEFT JOIN group_control_map " .
                 "ON group_control_map.product_id = products.id ";
@@ -735,9 +576,7 @@ sub GetSelectableProducts {
     PushGlobalSQLState();
     SendSQL($query);
     my @products = ();
-    while (MoreSQLData()) {
-        push @products,FetchOneColumn();
-    }
+    push(@products, FetchSQLData()) while MoreSQLData();
     PopGlobalSQLState();
     return (@products);
 }
@@ -745,50 +584,50 @@ sub GetSelectableProducts {
 # GetSelectableProductHash
 # returns a hash containing 
 # legal_products => an enterable product list
-# legal_components => the list of components of enterable products
-# components => a hash of component lists for each enterable product
+# legal_(components|versions|milestones) =>
+#   the list of components, versions, and milestones of enterable products
+# (components|versions|milestones)_by_product
+#    => a hash of component lists for each enterable product
+# Milestones only get returned if the usetargetmilestones parameter is set.
 sub GetSelectableProductHash {
-    my $query = "SELECT products.name, components.name " .
-                "FROM products " .
-                "LEFT JOIN components " .
-                "ON components.product_id = products.id " .
-                "LEFT JOIN group_control_map " .
-                "ON group_control_map.product_id = products.id ";
-    if (Param('useentrygroupdefault')) {
-        $query .= "AND group_control_map.entry != 0 ";
-    } else {
-        $query .= "AND group_control_map.membercontrol = " .
-                  CONTROLMAPMANDATORY . " ";
-    }
-    if ((defined @{$::vars->{user}{groupids}}) 
-        && (@{$::vars->{user}{groupids}} > 0)) {
-        $query .= "AND group_id NOT IN(" . 
-                   join(',', @{$::vars->{user}{groupids}}) . ") ";
-    }
-    $query .= "WHERE group_id IS NULL " .
-              "ORDER BY products.name, components.name";
+    # The hash of selectable products and their attributes that gets returned
+    # at the end of this function.
+    my $selectables = {};
+
+    my %products = GetSelectableProducts(1);
+
+    $selectables->{legal_products} = [sort values %products];
+
+    # Run queries that retrieve the list of components, versions,
+    # and target milestones (if used) for the selectable products.
+    my @tables = qw(components versions);
+    push(@tables, 'milestones') if Param('usetargetmilestone');
+
     PushGlobalSQLState();
-    SendSQL($query);
-    my @products = ();
-    my %components = ();
-    my %components_by_product = ();
-    while (MoreSQLData()) {
-        my ($product, $component) = FetchSQLData();
-        if (!grep($_ eq $product, @products)) {
-            push @products, $product;
-        }
-        if ($component) {
-            $components{$component} = 1;
-            push @{$components_by_product{$product}}, $component;
+    foreach my $table (@tables) {
+        # Why oh why can't we standardize on these names?!?
+        my $fld = ($table eq "components" ? "name" : "value");
+
+        my $query = "SELECT $fld, product_id FROM $table WHERE product_id IN " .
+                    "(" . join(",", keys %products) . ") ORDER BY $fld";
+        SendSQL($query);
+
+        my %values;
+        my %values_by_product;
+
+        while (MoreSQLData()) {
+            my ($name, $product_id) = FetchSQLData();
+            next unless $name;
+            $values{$name} = 1;
+            push @{$values_by_product{$products{$product_id}}}, $name;
         }
+
+        $selectables->{"legal_$table"} = [sort keys %values];
+        $selectables->{"${table}_by_product"} = \%values_by_product;
     }
     PopGlobalSQLState();
-    my @componentlist = (sort keys %components);
-    return {
-        legal_products => \@products,
-        legal_components => \@componentlist,
-        components => \%components_by_product,
-    };
+
+    return $selectables;
 }
 
 
@@ -827,7 +666,7 @@ sub CanSeeBug {
                || (($userid > 0) && 
                   (
                        ($assigned_to == $userid) 
-                    || ($qa_contact == $userid)
+                    || (Param('useqacontact') && $qa_contact == $userid)
                     || (($reporter == $userid) && $rep_access) 
                     || ($found_cc && $cc_access) 
                     || ($found_groups == $found_members)
@@ -889,24 +728,28 @@ sub Crypt {
 # Permissions must be rederived if ANY groups have a last_changed newer
 # than the profiles.refreshed_when value.
 sub ConfirmGroup {
-    my ($user) = (@_);
+    my ($user, $locked) = (@_);
     PushGlobalSQLState();
     SendSQL("SELECT userid FROM profiles, groups WHERE userid = $user " .
             "AND profiles.refreshed_when <= groups.last_changed ");
     my $ret = FetchSQLData();
     PopGlobalSQLState();
     if ($ret) {
-        DeriveGroup($user);
+        DeriveGroup($user, $locked);
     }
 }
 
 # DeriveGroup removes and rederives all derived group permissions for
-# the specified user.
+# the specified user.  If $locked is true, Bugzilla has already locked
+# the necessary tables as part of a larger transaction, so this function
+# shouldn't lock them again (since then tables not part of this function's
+# lock will get unlocked).
 sub DeriveGroup {
-    my ($user) = (@_);
+    my ($user, $locked) = (@_);
     PushGlobalSQLState();
 
-    SendSQL("LOCK TABLES profiles WRITE, user_group_map WRITE, group_group_map READ, groups READ");
+    SendSQL("LOCK TABLES profiles WRITE, user_group_map WRITE, group_group_map READ, groups READ")
+      unless $locked;
 
     # avoid races,  we are only as up to date as the BEGINNING of this process
     SendSQL("SELECT login_name, NOW() FROM profiles WHERE userid = $user");
@@ -1029,8 +872,8 @@ sub DBNameToIdAndCheck {
         return $result;
     }
 
-    $::vars->{'name'} = $name;
-    ThrowUserError("invalid_username");
+    ThrowUserError("invalid_username",
+                   { name => $name });
 }
 
 
@@ -1347,22 +1190,6 @@ sub SplitEnumType {
     return @result;
 }
 
-
-# This routine is largely copied from Mysql.pm.
-
-sub SqlQuote {
-    my ($str) = (@_);
-#     if (!defined $str) {
-#         confess("Undefined passed to SqlQuote");
-#     }
-    $str =~ s/([\\\'])/\\$1/g;
-    $str =~ s/\0/\\0/g;
-    # If it's been SqlQuote()ed, then it's safe, so we tell -T that.
-    trick_taint($str);
-    return "'$str'";
-}
-
-
 # UserInGroup returns information aboout the current user if no second 
 # parameter is specified
 sub UserInGroup {
@@ -1672,6 +1499,7 @@ sub FormatTimeUnit {
 # Constructs a format object from URL parameters. You most commonly call it 
 # like this:
 # my $format = GetFormat("foo/bar", $::FORM{'format'}, $::FORM{'ctype'});
+
 sub GetFormat {
     my ($template, $format, $ctype) = @_;
 
@@ -1687,11 +1515,28 @@ sub GetFormat {
     $template .= ($format ? "-$format" : "");
     $template .= ".$ctype.tmpl";
 
+    # Now check that the template actually exists. We only want to check
+    # if the template exists; any other errors (eg parse errors) will
+    # end up being detected laer.
+    eval {
+        Bugzilla->template->context->template($template);
+    };
+    # This parsing may seem fragile, but its OK:
+    # http://lists.template-toolkit.org/pipermail/templates/2003-March/004370.html
+    # Even if it is wrong, any sort of error is going to cause a failure
+    # eventually, so the only issue would be an incorrect error message
+    if ($@ && $@->info =~ /: not found$/) {
+        ThrowUserError("format_not_found", { 'format' => $format,
+                                             'ctype' => $ctype,
+                                           });
+    }
+
+    # Else, just return the info
     return
     {
         'template'    => $template ,
         'extension'   => $ctype ,
-        'ctype' => $::contenttypes->{$ctype} || "text/plain" ,
+        'ctype' => $::contenttypes->{$ctype} ,
     };
 }
 
@@ -1699,9 +1544,7 @@ sub GetFormat {
 
 use Bugzilla;
 
-$::BZ = Bugzilla->create();
-
-$::template = $::BZ->template();
+$::template = Bugzilla->template();
 
 $::vars = {};
 
