@@ -26,15 +26,16 @@ package Support::Files;
 # exclude_deps is a hash of arrays listing the files to be excluded
 # if a module is not available
 #
-@additional_files = ('syncshadowdb','processmail');
+@additional_files = ();
 %exclude_deps = (
     'XML::Parser' => ['importxml.pl'],
+    'Net::LDAP' => ['Bugzilla/Auth/LDAP.pm'],
 );
 
 
-# XXX - this file should be rewritten to use File::Find or similar
+# XXX - this file should really be rewritten to use File::Find or similar
 $file = '*';
-@files = (glob($file), glob('Bugzilla/*.pm'));
+@files = (glob($file), glob('Bugzilla/*.pm'), glob('Bugzilla/*/*.pm'));
 
 sub have_pkg {
     my ($pkg) = @_;
